/*
 * Authors: Zsolt Molnar <zsolt.molnar@cern.ch, http://www.zsoltmolnar.hu>
 *
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Test if the logic recognizes unknown protocols of wrongly formatted
 * protocol lists in the transfetParameters property of PrepareToGet.
 */
#include "lcg_util.h"
#include "gfal_api.h"
#include "gfal_dependencies.h"
#include "globus_dependencies.h"
#include <assert.h>
#include <errno.h>
#include <string.h>

#include "lcg_unit_testsuite.h"

#define __CALL_TEST(test) \
    printf("\nTest group: %s\n", #test); \
    res = (test); \
    if (res) \
        return res;

/* Which scenario we are playing */
typedef enum _protocol_list_scenario_type {
    E_UNSUPPORTED_PROTOCOL_SRC, /* Checking source unsupported protocol */
    E_UNSUPPORTED_PROTOCOL_DST, /* Checking destination unsupported protocol */
    E_EMPTY_LIST, /* Checking the empty protocol list */
    E_UNUSED
} _protocol_list_scenario_type_t;

/* The variable controls the actual scenario. The test function sets it,
 * then calls the function to be tested. Deep inside, the function will call
 * the (replaced) SOAP caller. The mock SOAP caller prepares different SRM
 * responses and return it, without the physical gSOAP/network operation.
 * The response content is controlled by this variable (the scenario). */
static _protocol_list_scenario_type_t _protocol_list_scenario = E_UNUSED;

/* The scenarios may have internal stages. For instance, in case of checking
 * unsupported protocol, the first stage: calling turlsfromsurl on the source,
 * second stage: calling turlsfromsurl on the destination */
typedef enum _protocol_list_scenario_stage {
    E_SCENARIO_STAGE_SRC,
    E_SCENARIO_STAGE_DST,
    E_SCENARIO_STAGE_UNUSED
} _protocol_list_scenario_stage_t;

static _protocol_list_scenario_stage_t
    _protocol_list_scenario_stage = E_SCENARIO_STAGE_UNUSED;

/* A fixture with a protocol list containing an unsupported protocol name */
static char* _unsupported_protocol[] = {"unsupported_protocol", NULL};

#define GFAL_RETURN_CODE_OK 0
#define GFAL_RETURN_CODE_ERROR -1

/* The dependency injection comes here. This function replaces the production
 * version SOAP calls so that no network operation, server, etc be required.
 * The function directly produces and returns the SRM response objects.
 *
 * This is the "SOAP call" for the srmPrepareToGet operation. */
static int _caller_play_scenario_gfal_turlsfromsurls(
    gfal_internal req, char *errbuf, int errbufsz)
{
    char* which_to_check = NULL;
    req->results_size = 1;

    switch(_protocol_list_scenario) {
    case E_EMPTY_LIST:
        LCG_TEST_ASSERT(strcmp(req->protocols[0], "gsiftp") == 0);
        LCG_TEST_EQUAL(NULL, req->protocols[1]);
        errno = 0;
        return GFAL_RETURN_CODE_OK;
    case E_UNSUPPORTED_PROTOCOL_SRC:
        which_to_check = (_protocol_list_scenario_stage == E_SCENARIO_STAGE_SRC ?
                          _unsupported_protocol[0] : "gsiftp");
    case E_UNSUPPORTED_PROTOCOL_DST:
        if (!which_to_check) {
            which_to_check = (_protocol_list_scenario_stage == E_SCENARIO_STAGE_DST ?
                             _unsupported_protocol[0] : "gsiftp");
        }
        LCG_TEST_ASSERT(strcmp(req->protocols[0], which_to_check) == 0);
        LCG_TEST_EQUAL(req->protocols[1], NULL);
        ++_protocol_list_scenario_stage;
        errno = EOPNOTSUPP;
        return GFAL_RETURN_CODE_ERROR;
    default:
        assert(0);
    }

    return GFAL_RETURN_CODE_ERROR;
}

static void _start_scenario(_protocol_list_scenario_type_t s)
{
    _protocol_list_scenario = s;
    _protocol_list_scenario_stage = E_SCENARIO_STAGE_SRC;
}

static int _caller_play_scenario_gfal_ls(
    gfal_internal req, char *errbuf, int errbufsz)
{
    return GFAL_RETURN_CODE_OK;
}

static int _caller_play_scenario_gfal_get_results(
    gfal_internal req, gfal_filestatus **results)
{
    *results = malloc(sizeof(gfal_filestatus));
    (*results)[0].status = 0;
    (*results)[0].turl = malloc(sizeof(char));
    (*results)[0].turl[0]= 0;
    return 1;
}

static int _caller_play_scenario_gfal_set_xfer_done(
    gfal_internal gfal, char *errbuf, int errbufsz)
{
    errno = 0;
    return 0;
}

static int _caller_play_scenario_globus_copyfilex(
    char *src_file, char *dest_file, int nbstreams, char *errbuf,
    int errbufsz, int timeout, int verbose)
{
    errno = 0;
    return 0;
}

static int _caller_play_scenario_globus_getfilesizet(
    char *file, GFAL_LONG64 *size, char *errbuf, int errbufsz, int timeout)
{
    errno = 0;
    *size = 1;
    return 0;
}

static int _caller_play_scenario_gfal_register_file (
    const char *lfn, const char *guid, const char *surl, mode_t mode,
    GFAL_LONG64 size, int bool_createonly, char *errbuf, int errbufsz)
{
    errno = 0;
    return 0;
}

static char * _caller_play_scenario_gfal_guidforpfn (
    const char *pfn, char *errbuf, int errbufsz)
{
    static const char* v = "fake_guid";
    char *ret = malloc(strlen(v) + 1);
    return strcpy(ret, v);
}
static int _caller_play_scenario_gfal_init (gfal_request req, gfal_internal *gfal, char *errbuf, int errbufsz)
{
	if ((*gfal = (gfal_internal) malloc (sizeof (struct gfal_internal_))) == NULL) {
	        errno = ENOMEM;
	        return (-1);
	    }

	memset (*gfal, 0, sizeof (struct gfal_internal_));
	memcpy (*gfal, req, sizeof (struct gfal_request_));
	/* Use default SRM timeout if not specified in request */
	if (!(*gfal)->timeout)
		(*gfal)->timeout = gfal_get_timeout_srm ();

	(*gfal)->setype = TYPE_SE;
	return (0);
}

/* Inject the dependencies. Replace the SOAP callers to the test ones. */
static void _setup_callers()
{
    lcg_gfal_functions.call_gfal_turlsfromsurls = _caller_play_scenario_gfal_turlsfromsurls;
    lcg_gfal_functions.call_gfal_ls = _caller_play_scenario_gfal_ls;
    lcg_gfal_functions.call_gfal_get_results = _caller_play_scenario_gfal_get_results;
    lcg_gfal_functions.call_gfal_set_xfer_done = _caller_play_scenario_gfal_set_xfer_done;
    lcg_gfal_functions.call_gfal_register_file = _caller_play_scenario_gfal_register_file;
    lcg_gfal_functions.call_gfal_guidforpfn = _caller_play_scenario_gfal_guidforpfn;
    lcg_gfal_functions.call_gfal_init = _caller_play_scenario_gfal_init;

    lcg_globus_functions.call_globus_copyfilex = _caller_play_scenario_globus_copyfilex;
    lcg_globus_functions.call_globus_getfilesizet = _caller_play_scenario_globus_getfilesizet;
}

static char *mock_src_file = "gsiftp://src";
static char *mock_src_file_srm = "srm://fake2.cern.ch:8443/srm/managerv2?SFN=/src";
static char *mock_dst_file_srm = "srm://fake.cern.ch:8443/srm/managerv2?SFN=/dst";
static char *mock_vo = "fake_vo";
static int mock_nbfiles = 0;
static int mock_nobdii = 1;
static const char *mock_srm_endpoint = "httpg://srm-v2.cern.ch:8443/srm/managerv2";
static int mock_desiredpintime = 10;
static char *mock_spacetokendesc = NULL;
static char ** mock_reqtoken = NULL;
static struct srmv2_pinfilestatus * mock_pinfilestatus[3];
static int mock_timeout = 100;
static int mock_nbstreams = 1;
static char *mock_conf_file = NULL;
static int mock_insecure = 1;
static int mock_verbose = 0;
static enum gfal_cksm_type mock_cksmtype = GFAL_CKSM_NONE;
static int errbufsz = 1024;
static char errbuf[1024];
static char *mock_relative_path = NULL;
static char **mock_actual_surl = NULL;

static char* _test_lcg_cp5()
{
    int res = 1;
    /* Test empty list */
    _start_scenario(E_EMPTY_LIST);
    errno = 0;

    res = lcg_cp5 (mock_src_file_srm, mock_dst_file_srm, TYPE_SRMv2, TYPE_SRMv2,
        TYPE_SRMv2,
        mock_nobdii, mock_vo, mock_nbstreams, mock_conf_file, mock_insecure,
        mock_verbose, mock_timeout, mock_spacetokendesc, mock_spacetokendesc,
        mock_cksmtype,
        NULL, NULL,
        errbuf, errbufsz);

    LCG_TEST_EQUAL(0, errno);
    LCG_TEST_EQUAL(0, res);

    /* Test the unsupported protocol list */
    _start_scenario(E_UNSUPPORTED_PROTOCOL_SRC);

    res = lcg_cp5 (mock_src_file_srm, mock_dst_file_srm, TYPE_SRMv2, TYPE_SRMv2,
        TYPE_SRMv2,
        mock_nobdii, mock_vo, mock_nbstreams, mock_conf_file, mock_insecure,
        mock_verbose, mock_timeout, mock_spacetokendesc, mock_spacetokendesc,
        mock_cksmtype,
        _unsupported_protocol, NULL,
        errbuf, errbufsz);

    LCG_TEST_EQUAL(EOPNOTSUPP, errno);
    LCG_TEST_EQUAL(-1, res);

    errno = 0;
    _start_scenario(E_UNSUPPORTED_PROTOCOL_DST);

    res = lcg_cp5 (mock_src_file_srm, mock_dst_file_srm, TYPE_SRMv2, TYPE_SRMv2,
        TYPE_SRMv2,
        mock_nobdii, mock_vo, mock_nbstreams, mock_conf_file, mock_insecure,
        mock_verbose, mock_timeout, mock_spacetokendesc, mock_spacetokendesc,
        mock_cksmtype,
        NULL, _unsupported_protocol,
        errbuf, errbufsz);

    LCG_TEST_EQUAL(EOPNOTSUPP, errno);
    LCG_TEST_EQUAL(-1, res);

    return 0x0;
}

static char* _test_lcg_cr5()
{
    int res = 1;
    char *mock_guid = "fake_guid";
    char *mock_actual_guid = NULL;
    char *mock_lfn_arg = NULL;

    /* Test empty list */
    _start_scenario(E_EMPTY_LIST);
    errno = 0;

    res = lcg_cr5 (mock_src_file, mock_dst_file_srm, mock_guid, mock_lfn_arg,
        TYPE_NONE, TYPE_SRMv2, mock_nobdii, mock_vo, mock_relative_path,
        mock_nbstreams, mock_conf_file, mock_insecure, mock_verbose,
        mock_timeout, mock_spacetokendesc, mock_actual_guid, mock_cksmtype,
        NULL, /* Empty protocol list */
        errbuf, errbufsz);

    LCG_TEST_EQUAL(0, errno);
    LCG_TEST_EQUAL(0, res);

    /* Test the unsupported protocol list */
    _start_scenario(E_UNSUPPORTED_PROTOCOL_SRC);
    errno = 0;

    res = lcg_cr5 (mock_src_file, mock_dst_file_srm, mock_guid, mock_lfn_arg,
        TYPE_NONE, TYPE_SRMv2, mock_nobdii, mock_vo, mock_relative_path,
        mock_nbstreams, mock_conf_file, mock_insecure, mock_verbose,
        mock_timeout, mock_spacetokendesc, mock_actual_guid,
        mock_cksmtype,
        _unsupported_protocol, /* Unsupported protocol list */
        errbuf, errbufsz);

    LCG_TEST_EQUAL(EOPNOTSUPP, errno);
    LCG_TEST_EQUAL(-1, res);

    return 0x0;
}

static char* _test_lcg_rep5()
{
    int res = 1;
    /* Test empty list */
    _start_scenario(E_EMPTY_LIST);
    errno = 0;

    res = lcg_rep5 (mock_src_file_srm, mock_dst_file_srm, TYPE_SRMv2, TYPE_SRMv2,
        TYPE_SRMv2, mock_nobdii, mock_vo, mock_relative_path, mock_nbstreams,
        mock_conf_file, mock_insecure, mock_verbose, mock_timeout,
        mock_spacetokendesc, mock_spacetokendesc, mock_actual_surl,
        mock_cksmtype,
        NULL, NULL,
        errbuf, errbufsz);

    LCG_TEST_EQUAL(0, errno);
    LCG_TEST_EQUAL(0, res);

    /* Test the unsupported protocol list */
    _start_scenario(E_UNSUPPORTED_PROTOCOL_SRC);
    errno = 0;

    res = lcg_rep5 (mock_src_file_srm, mock_dst_file_srm, TYPE_SRMv2, TYPE_SRMv2,
        TYPE_SRMv2, mock_nobdii, mock_vo, mock_relative_path, mock_nbstreams,
        mock_conf_file, mock_insecure, mock_verbose, mock_timeout,
        mock_spacetokendesc, mock_spacetokendesc, mock_actual_surl,
        mock_cksmtype,
        _unsupported_protocol, NULL,
        errbuf, errbufsz);

    LCG_TEST_EQUAL(EOPNOTSUPP, errno);
    LCG_TEST_EQUAL(-1, res);

    errno = 0;
    _start_scenario(E_UNSUPPORTED_PROTOCOL_DST);

    res = lcg_rep5 (mock_src_file_srm, mock_dst_file_srm, TYPE_SRMv2, TYPE_SRMv2,
        TYPE_SRMv2, mock_nobdii, mock_vo, mock_relative_path, mock_nbstreams,
        mock_conf_file, mock_insecure, mock_verbose, mock_timeout,
        mock_spacetokendesc, mock_spacetokendesc, mock_actual_surl,
        mock_cksmtype,
        NULL, _unsupported_protocol,
        errbuf, errbufsz);

    LCG_TEST_EQUAL(EOPNOTSUPP, errno);
    LCG_TEST_EQUAL(-1, res);

    return 0x0;
}

char * lcg_test__protocol_list_handling()
{
    char * res = 0x0;
    _setup_callers();
    __CALL_TEST(_test_lcg_cp5());
    __CALL_TEST(_test_lcg_cr5());
    __CALL_TEST(_test_lcg_rep5());
    return 0x0;
}

#undef __CALL_TEST
